import json
from typing import Any

from .types import EntityIdentifier
from .types import MeroxaApiResponse
from .utils import api_response
from .utils import ComplexEncoder

CONNECTORS_BASE_PATH = "/v1/connectors"


class ConnectorsResponse(MeroxaApiResponse):
    def __init__(
        self,
        id: str,
        config: dict[str, str],
        created_at: str,
        metadata: dict[str, str],
        name: str,
        resource_name: str,
        pipeline_name: str,
        resource_id: str,
        resource_uuid: str,
        pipeline_id: str,
        streams: dict[str, str],
        state: str,
        type: str,
        updated_at: str,
        uuid: str,
        collection: str,
        trace: str = None,
        environment: EntityIdentifier = None,
    ) -> None:
        self.id = id
        self.config = config
        self.created_at = created_at
        self.metadata = metadata
        self.name = name
        self.resource_name = resource_name
        self.pipeline_name = pipeline_name
        self.resource_id = resource_id
        self.resource_uuid = resource_id
        self.pipeline_id = pipeline_id
        self.streams = streams
        self.state = state
        self.type = type
        self.updated_at = updated_at
        self.uuid = uuid
        self.collection = collection
        self.trace = trace
        self.environment = environment
        super().__init__()


class CreateConnectorParams:
    def __init__(
        self,
        resource_name: str,
        pipeline_name: str,
        name: str = None,
        config: dict[str, str] = None,
        metadata: dict[str, str] = None,
        connector_type: str = None,
        input: str = None,
    ) -> None:
        self._resource_name = resource_name
        self._pipeline_name = pipeline_name
        self._name = name
        self._config = config
        self._metadata = metadata
        self._connector_type = connector_type
        self._input = input

    def repr_json(self):
        return dict(
            resource_name=self._resource_name,
            pipeline_name=self._pipeline_name,
            name=self._name,
            config=self._config,
            metadata=self._metadata,
            connector_type=self._connector_type,
            input=self._input,
        )


class UpdateConnectorParams:
    def __init__(
        self,
        name: str = None,
        config: dict[str, Any] = None,
    ) -> None:
        self._name = name
        self._config = config

    def repr_json(self):
        return dict(
            config=self._config,
            name=self._name,
        )


class Connectors:
    def __init__(self, session) -> None:
        self._session = session

    @api_response(ConnectorsResponse)
    async def get(self, name_or_id: str):
        async with self._session.get(
            CONNECTORS_BASE_PATH + "/{}".format(name_or_id)
        ) as resp:
            return await resp.text()

    @api_response(ConnectorsResponse)
    async def list(self):
        async with self._session.get(CONNECTORS_BASE_PATH) as resp:
            return await resp.text()

    async def delete(self, name_or_id: str):
        async with self._session.delete(
            CONNECTORS_BASE_PATH + "/{}".format(name_or_id)
        ) as resp:
            return await resp.text()

    @api_response(ConnectorsResponse)
    async def create(self, create_connector_parameters: CreateConnectorParams):
        async with self._session.post(
            CONNECTORS_BASE_PATH,
            data=json.dumps(
                create_connector_parameters.repr_json(), cls=ComplexEncoder
            ),
        ) as resp:
            return await resp.text()

    @api_response(ConnectorsResponse)
    async def update(
        self, name_or_id: str, update_connector_parameters: UpdateConnectorParams
    ):
        async with self._session.post(
            CONNECTORS_BASE_PATH + "/{}".format(name_or_id),
            json=json.dumps(
                update_connector_parameters.repr_json(), cls=ComplexEncoder
            ),
        ) as resp:
            return await resp.text()



import json

from .pipelines import PipelineIdentifiers
from .types import ApplicationResource
from .types import EntityIdentifier
from .types import MeroxaApiResponse
from .utils import api_response
from .utils import ComplexEncoder

APPLICATIONS_BASE_PATH = "/v1/applications"


class ApplicationResponse(MeroxaApiResponse):
    def __init__(
        self,
        uuid: str,
        name: str,
        language: str,
        status: dict,
        pipeline: dict,
        created_at: str,
        updated_at: str,
        git_sha: str = None,
        connectors: list[EntityIdentifier] = None,
        functions: list[EntityIdentifier] = None,
        resources: list[ApplicationResource] = None,
    ) -> None:
        self.uuid = uuid
        self.name = name
        self.language = language
        self.status = status
        self.pipeline = pipeline
        self.created_at = created_at
        self.updated_at = updated_at
        self.git_sha = git_sha
        self.connectors = connectors
        self.functions = functions
        self.resources = resources
        super().__init__()


class CreateApplicationParams:
    def __init__(
        self,
        name: str,
        language: str,
        git_sha: str,
        pipeline: PipelineIdentifiers,
    ) -> None:
        self._name = name
        self._language = language
        self._git_sha = git_sha
        self._pipeline = pipeline

    def repr_json(self):
        return dict(
            name=self._name,
            language=self._language,
            git_sha=self._git_sha,
            pipeline=self._pipeline,
        )


class Applications:
    def __init__(self, session) -> None:
        self._session = session

    @api_response(ApplicationResponse)
    async def get(self, name_or_uuid: str):

        async with self._session.get(
            APPLICATIONS_BASE_PATH + "/{}".format(name_or_uuid)
        ) as resp:
            return await resp.text()

    @api_response(ApplicationResponse)
    async def list(self):

        async with self._session.get(APPLICATIONS_BASE_PATH) as resp:
            return await resp.text()

    async def delete(self, name_or_uuid: str):
        async with self._session.delete(
            APPLICATIONS_BASE_PATH + "/{}".format(name_or_uuid)
        ) as resp:
            return await resp.text()

    @api_response(ApplicationResponse)
    async def create(self, create_application_parameters: CreateApplicationParams):
        async with self._session.post(
            APPLICATIONS_BASE_PATH,
            data=json.dumps(
                create_application_parameters.repr_json(), cls=ComplexEncoder
            ),
        ) as resp:
            return await resp.text()



import json
from json import JSONDecodeError
from typing import Type

from .types import MeroxaApiResponse


class ComplexEncoder(json.JSONEncoder):
    def default(self, obj):
        if hasattr(obj, "repr_json"):
            return obj.repr_json()
        else:
            return json.JSONEncoder.default(self, obj)


class ErrorResponse(object):
    def __init__(self, code: str, message: str, details=None) -> None:
        self.code = code
        self.message = message
        self.details = details

    def __repr__(self):
        details = ""
        if self.details and len(self.details) > 0:
            details = f"; {len(self.details)} detail(s) provided\n"
            count = 1
            for i in self.details:
                joined = ". ".join(self.details[i])
                iter = f"{count}. {i}: {joined}\n"
                details = details + iter
                count += 1

        return f"{self.message} {details}"


def parse_error_message(error):
    try:
        return ErrorResponse(**json.loads(error))
    except JSONDecodeError or TypeError:
        split = error.split("\n", 1)
        return ErrorResponse("Error", split[0])


def api_response(return_type: Type[MeroxaApiResponse]):
    def inner(func):
        async def wrapper(*args, **kwargs):
            res = await func(*args, **kwargs)
            try:
                parsed = json.loads(res)
                if isinstance(parsed, list):
                    return None, [return_type(**par) for par in parsed]
                return None, return_type(**json.loads(res))
            except (JSONDecodeError, TypeError):
                return parse_error_message(res), None

        return wrapper

    return inner



from .types import MeroxaApiResponse
from .utils import api_response


class UserResponse(MeroxaApiResponse):
    def __init__(
        self,
        uuid: str,
        email: str,
        given_name: str,
        family_name: str,
        email_verified: bool,
        picture: str,
        last_login: str,
        features: list[str],
        username=None,
    ) -> None:
        self.uuid = uuid
        self.email = email
        self.given_name = given_name
        self.family_name = family_name
        self.email_verified = email_verified
        self.picture = picture
        self.last_login = last_login
        self.features = features
        self.username = username
        super().__init__()


class Users:
    def __init__(self, session) -> None:
        self._session = session

    @api_response(UserResponse)
    async def me(self):
        async with self._session.get("/v1/users/me") as resp:
            return await resp.text()
